﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Media;
using Infragistics.Controls.Charts;
using System.Windows.Input;
using System.Windows.Shapes;
using System.Windows.Controls.Primitives;
using System.Windows.Controls;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Windows.Data;

namespace DataChart.Utilities
{
    #region Chart crosshair behaviors
    public class ChartCrosshairBehaviors : DependencyObject
    {
        #region Chart Crosshair Visibility Behavior

        internal const string CrosshairVisibilityPropertyName = "CrosshairVisibility";
        public static readonly DependencyProperty CrosshairVisibilityProperty =
            DependencyProperty.RegisterAttached(CrosshairVisibilityPropertyName,
            typeof(ChartCrosshairVisibilityBehavior), typeof(ChartCrosshairBehaviors),
            new PropertyMetadata(null, (o, e) => OnChartCrosshairsBehaviorChanged(o as XamDataChart,
                    e.OldValue as ChartCrosshairVisibilityBehavior,
                    e.NewValue as ChartCrosshairVisibilityBehavior)));

        public static ChartCrosshairVisibilityBehavior GetCrosshairVisibility(DependencyObject target)
        {
            return target.GetValue(CrosshairVisibilityProperty) as ChartCrosshairVisibilityBehavior;
        }
        public static void SetCrosshairVisibility(DependencyObject target, ChartCrosshairVisibilityBehavior behavior)
        {
            target.SetValue(CrosshairVisibilityProperty, behavior);
        }

        private static void OnChartCrosshairsBehaviorChanged(XamDataChart chart, ChartCrosshairVisibilityBehavior oldValue, ChartCrosshairVisibilityBehavior newValue)
        {
            if (chart == null)
            {
                return;
            }
            if (oldValue != null)
            {
                oldValue.OnDetach(chart);
            }
            if (newValue != null)
            {
                newValue.OnAttach(chart);
            }
        }
        #endregion
    }
    public class ChartCrosshairVisibilityBehavior
    {
        public ChartCrosshairVisibilityBehavior()
        {
            _invisibleStyle = new Style(typeof(Line));
            _invisibleStyle.Setters.Add(new Setter(Shape.StrokeProperty, new SolidColorBrush(Colors.Transparent)));
        }

        #region Properties
        private bool _first = true;
        private Line _vertical;
        private Line _horizontal;
        private readonly Style _invisibleStyle;
        public bool ShowHorizontalCrosshair { get; set; }
        public bool ShowVerticalCrosshair { get; set; }
        #endregion

        #region Methods
        private void MakeInvisible(XamDataChart chart, Line crosshairLine)
        {
            crosshairLine.Style = _invisibleStyle;
        }
        private void MakeVisible(XamDataChart chart, Line crosshairLine)
        {
            if (chart == null || crosshairLine == null)
            {
                return;
            }
            crosshairLine.Style = chart.CrosshairLineStyle;
        }
        #endregion

        #region Event Handlers
        public void OnAttach(XamDataChart chart)
        {
            chart.MouseEnter += OnChartMouseEnter;
        }
        public void OnDetach(XamDataChart chart)
        {
            chart.MouseEnter -= OnChartMouseEnter;
            MakeVisible(chart, _vertical);
            MakeVisible(chart, _horizontal);
            _vertical = null;
            _horizontal = null;
            _first = true;
        }

        void OnChartMouseEnter(object sender, MouseEventArgs e)
        {
            XamDataChart chart = sender as XamDataChart;
            if (_first && chart != null)
            {
                
                _first = false;
                var crosshairs =
                    from line in chart.VisualDescendants()
                    where line is Line && (line as Line).Style
                          == chart.CrosshairLineStyle
                    select line as Line;
                List<Line> lines = crosshairs.ToList();
                if (lines.Count < 2)
                {
                    _first = true;
                    chart.MouseMove += OnChartMouseEnter;
                    return;
                }
                bool uninitialized = lines[0].X1 == 0.0 && lines[0].X2 == 0.0 &&
                                     lines[1].X1 == 0.0 && lines[1].X2 == 0.0;
                if (uninitialized)
                {
                    _first = true;
                    chart.MouseMove += OnChartMouseEnter;
                    return;
                }
                if (lines[0].X1 == lines[0].X2)
                {
                    _vertical = lines[0];
                    _horizontal = lines[1];
                }
                else
                {
                    _vertical = lines[1];
                    _horizontal = lines[0];
                }

                if (!this.ShowHorizontalCrosshair)
                {
                    MakeInvisible(chart, _horizontal);
                }
                if (!this.ShowVerticalCrosshair)
                {
                    MakeInvisible(chart, _vertical);
                }
                chart.MouseMove -= OnChartMouseEnter;
            }
        }
        #endregion
    }

    public static class ChartExtension
    {
        public static IEnumerable<DependencyObject> VisualDescendants(this DependencyObject current)
        {
            yield return current;
            for (int i = 0; i < VisualTreeHelper.GetChildrenCount(current); i++)
            {
                DependencyObject child = VisualTreeHelper.GetChild(current, i);
                foreach (DependencyObject sub in child.VisualDescendants())
                {
                    yield return sub;
                }
            }
        }
    }
    #endregion End chart crosshair behaviors

    #region Chart tooltip behavior
    public class ChartTooltipBehaviors : DependencyObject
    {
        public static readonly DependencyProperty CursorTooltipProperty =
            DependencyProperty.RegisterAttached("CursorTooltip",
            typeof(CursorTooltipBehavior), typeof(ChartTooltipBehaviors),
            new PropertyMetadata(null,
                (o, e) => CursorTooltipChanged(
                    o as XamDataChart,
                    e.OldValue as CursorTooltipBehavior,
                    e.NewValue as CursorTooltipBehavior)));

        public static CursorTooltipBehavior GetCursorTooltip(
            DependencyObject target)
        {
            return target.GetValue(CursorTooltipProperty)
                as CursorTooltipBehavior;
        }

        public static void SetCursorTooltip(
            DependencyObject target, CursorTooltipBehavior behavior)
        {
            target.SetValue(CursorTooltipProperty, behavior);
        }

        private static void CursorTooltipChanged(
            XamDataChart chart,
            CursorTooltipBehavior oldValue,
            CursorTooltipBehavior newValue)
        {
            if (chart == null)
            {
                return;
            }
            if (oldValue != null)
            {
                oldValue.OnDetach(chart);
            }
            if (newValue != null)
            {
                newValue.OnAttach(chart);
            }
        }
    }

    public class SeriesItemInfo
        : INotifyPropertyChanged
    {
        private Series _series;
        public Series Series
        {
            get { return _series; }
            set
            {
                _series = value;
                if (PropertyChanged != null)
                {
                    PropertyChanged(
                        this,
                        new PropertyChangedEventArgs("Series"));
                }
            }
        }

        private object _item;
        public object Item
        {
            get { return _item; }
            set
            {
                _item = value;
                if (PropertyChanged != null)
                {
                    PropertyChanged(
                        this,
                        new PropertyChangedEventArgs("Item"));
                }
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;
    }

    public class SeriesItemInfoCollection
        : ObservableCollection<SeriesItemInfo>
    {
        public void UpdateSeriesItem(
            Series series,
            object item)
        {
            bool found = false;
            var indexes = from curr in this
                          where curr.Series == series
                          select this.IndexOf(curr);
            foreach (int index in indexes)
            {
                found = true;
                this[index].Item = item;
            }

            if (!found)
            {
                this.Add(
                    new SeriesItemInfo()
                    {
                        Series = series,
                        Item = item
                    });
            }
        }
    }

    public class CursorTooltipBehavior
    {
        private bool _isOverChart = false;
        private Popup _popup = new Popup();

        private ContentControl _content =
            new ContentControl();
        private Panel _container;
        private XamDataChart _owner = null;

        private DataTemplate _tooltipTemplate;

        private SeriesItemInfoCollection _items
            = new SeriesItemInfoCollection();

        public DataTemplate TooltipTemplate
        {
            get { return _tooltipTemplate; }
            set
            {
                _tooltipTemplate = value;
                _content.ContentTemplate = _tooltipTemplate;
            }
        }

        protected bool IsOverChart
        {
            get { return _isOverChart; }
            set
            {
                bool last = _isOverChart;
                _isOverChart = value;
                if (_isOverChart && !last)
                {
                    ShowPopup();
                }
                if (!_isOverChart && last)
                {
                    HidePopup();
                }
            }
        }

        private void HidePopup()
        {
            _popup.IsOpen = false;
        }

        private void ShowPopup()
        {
            _popup.Placement = PlacementMode.Absolute;
            _popup.IsOpen = true;
        }

        public void OnAttach(XamDataChart chart)
        {
            if (_owner != null)
            {
                OnDetach(_owner);
            }
            chart.MouseLeave += Chart_MouseLeave;
            chart.MouseMove += Chart_MouseMove;
            chart.SeriesCursorMouseMove +=
                Chart_SeriesCursorMouseMove;

            _popup.IsOpen = false;
            _popup.Child = _content;
            _content.ContentTemplate = TooltipTemplate;
            _content.Content = _items;
            if (chart.Parent != null && chart.Parent is Panel)
            {
                _container = chart.Parent as Panel;
                _container.Children.Add(_popup);
            }
        }

        public void OnDetach(XamDataChart chart)
        {
            if (_owner != chart)
            {
                return;
            }
            chart.MouseLeave -= Chart_MouseLeave;
            chart.MouseMove -= Chart_MouseMove;
            chart.SeriesCursorMouseMove -=
                Chart_SeriesCursorMouseMove;
            IsOverChart = false;
            _items.Clear();
            _owner = null;
        }

        void Chart_MouseMove(object sender, MouseEventArgs e)
        {
            double x = e.GetPosition(sender as XamDataChart).X + (sender as XamDataChart).PointToScreen(new Point(0, 0)).X;
            double y = e.GetPosition(sender as XamDataChart).Y + (sender as XamDataChart).PointToScreen(new Point(0, 0)).Y;
            Point p = new Point(x,y);
            SetPopupOffsets(p);
            IsOverChart = true;
        }

        private void SetPopupOffsets(Point point)
        {
            _popup.VerticalOffset = point.Y;
            _popup.HorizontalOffset = point.X + 10;
        }

        void Chart_SeriesCursorMouseMove(
            object sender, ChartCursorEventArgs e)
        {
            if (e.Series != null && e.Item != null)
            {
                _items.UpdateSeriesItem(e.Series, e.Item);
            }
        }

        void Chart_MouseLeave(
            object sender, MouseEventArgs e)
        {
            IsOverChart = false;
        }
    }
    #endregion End chart tooltip behavior

    #region Converters
    public class MyConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            if (value.ToString() == "Actual Measures" || value.ToString() == "Actual Price")
            {
                return Visibility.Visible;
            }
            else
                return Visibility.Collapsed;
        }

        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }

    public class NumericConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            double dVal = NumberParser.ParseDouble(value, double.NaN);
            if (dVal == double.NaN)
                return value;
            else
                return dVal.ToString("#0.0000");
        }

        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }

    public class NumericPercentConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            double dVal = NumberParser.ParseDouble(value, double.NaN);
            if (dVal == double.NaN)
                return value;
            else
                return (dVal * 100).ToString("#0.0000") + "%";
        }

        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }

    public class NumberParser
    {
        #region NumberParser.ParseDouble Members

        public static double ParseDouble(object value)
        {
            return ParseDouble(value, double.NaN);
        }

        public static double ParseDouble(object value, double valueOnError)
        {
            //If null, then return error value
            if (Object.Equals(value, null))
                return valueOnError;

            //If a double, then cast and return
            if (value.GetType().Equals(typeof(double)))
                return (double)value;

            double theValue = double.NaN;
            if (IsNumeric(value.ToString(), out theValue))
                return theValue;
            else
                return valueOnError;
        }

        #endregion

        public static bool IsNumeric(string sValue, out double dfOut)
        {
            //check whether string argument is numeric

            bool bOut = Double.TryParse(sValue,
                System.Globalization.NumberStyles.Any,
                System.Globalization.NumberFormatInfo.InvariantInfo, out dfOut);

            return bOut;
        }
    }

    public class DateConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            DateTime dt = DateTime.MinValue;
            if (value != null && DateTime.TryParse(value.ToString(), out dt))
                return dt.ToString("MM/dd/yyyy");
            else
                return value;
        }

        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }
    #endregion


}
